#include "format.h"

#include <sys/types.h>	/* types, inet_ntoa */
#include <sys/socket.h>	/* types, inet_ntoa */
#include <arpa/inet.h>	/* types, inet_ntoa */
#include <stdio.h>	/* *printf */
#include <string.h>	/* strlen */




static int
sockaddr_unknown_to_str(const struct sockaddr *addr, socklen_t addrlen,
		char *buf, size_t buf_len)
{
	int p, i;
	p = snprintf(buf, buf_len, "family=%u ", addr->sa_family);
	for (i = 0; i < (int) addrlen && p < buf_len; i++, p += 2) {
		snprintf(buf + p, buf_len - p, "%02x",
				(unsigned int) addr->sa_data[i]);
	}
	return p;
}



static int
sockaddr_inet_to_str(const struct sockaddr *addr, socklen_t addrlen,
		char *buf, size_t buf_len)
{
	struct sockaddr_in *inet;
	char *ip_addr;

	inet = (struct sockaddr_in *) addr;
	ip_addr = inet_ntoa(inet->sin_addr);

	return snprintf(buf, buf_len, "%s:%u",
			ip_addr, ntohs(inet->sin_port));
}



static int
sockaddr_inet6_to_str(const struct sockaddr *addr, socklen_t addrlen,
		char *buf, size_t buf_len)
{
	struct sockaddr_in6 *inet;
	int len;

	inet = (struct sockaddr_in6 *) addr;
	inet_ntop(AF_INET6, &(inet->sin6_addr), buf, buf_len);
	len = (int) strlen(buf);
	len += snprintf(buf + len, buf_len - len, "/%d", ntohs(inet->sin6_port));

	return len;
}



int
sockaddr_to_str(char *buf, size_t buf_len,
		const struct sockaddr *addr, socklen_t addrlen)
{
	int len;
	if (addr == NULL) {
		if (addrlen > 0) {
			buf[0] = '\0';
		}
		return 0;
	}
	switch (addr->sa_family) {
		case AF_INET:
			len = sockaddr_inet_to_str(addr, addrlen,
					buf, buf_len);
			break;
		case AF_INET6:
			len = sockaddr_inet6_to_str(addr, addrlen,
					buf, buf_len);
			break;
		default:
			len = sockaddr_unknown_to_str(addr, addrlen,
					buf, buf_len);
			break;
	}
	buf[buf_len - 1] = '\0';
	return len;
}
