#include "match.h"
#include "log.h"
#include "internal.h"

#include <string.h>	/* strcmp, strncpy, strtok */
#ifndef NO_POSIX
#include <fnmatch.h>	/* fnmatch */
#endif
#include <sys/socket.h>	/* getnameinfo */
#include <netdb.h>	/* getnameinfo */
#include <dlfcn.h>	/* dlsym */

#define ADDR_LEN	64



static void
drop_port(char *addr)
{
	strtok(addr, ":/");
}



static int
smart_compare(const char *rule, const char *cand, int flags)
{
#ifndef NO_POSIX
	if (rule[0] == '~') {
		return fnmatch(rule + 1, cand, flags) == 0;
	} else if (strchr(rule, '*') != NULL) {
		return fnmatch(rule, cand, flags) == 0;
	} else {
		return strcmp(rule, cand) == 0;
	}
#else
	return strcmp(rule, cand) == 0;
#endif
}



int
match_addr(const char *rule, const char *cand)
{
	char a[ADDR_LEN + 1];
	char b[ADDR_LEN + 1];

	a[ADDR_LEN] = b[ADDR_LEN] = '\0';
	strncpy(a, rule, ADDR_LEN);
	strncpy(b, cand, ADDR_LEN);
	drop_port(a);
	drop_port(b);

	return smart_compare(a, b, FNM_PERIOD);
}



int
match_reverse_addr(const char *rule, const char *addr_str,
		const struct sockaddr *addr, socklen_t addr_len)
{
	char host[NI_MAXHOST];
	int r;

	internal_set();
	r = getnameinfo(addr, addr_len, host, sizeof(host), NULL, 0, 0);
	internal_unset();
	if (r == 0) {
		log_debug("  reversed \"%s\" to \"%s\"", addr_str, host);
		return match_addr(rule, host);
	} else {
		log_debug("  reverse getnameinfo for \"%s\" failed", addr_str);
		return 0;
	}
}



int
match_node(const char *rule, const char *cand)
{
	return smart_compare(rule, cand, 0);
}



int
match_service(const char *rule, const char *cand)
{
	return smart_compare(rule, cand, 0);
}
