#include "rules.h"
#include "config.h"
#include "log.h"
#include "match.h"

#include <stdlib.h>	/* free */



static int
match(struct resolve_rule *rule, const char *node, const char *service)
{
	int match = 0;

	log_debug("rule: (%s / %s) -- resolve: (%s / %s)",
			rule->node_pattern ? : "-",
			rule->service_pattern ? : "-",
			node ? : "-",
			service ? : "-");

	if (rule->node_pattern && node) {
		if (match_node(rule->node_pattern, node)) {
			match = 1;
		} else {
			return 0;
		}
	}
	if (rule->service_pattern && node) {
		if (match_service(rule->service_pattern, service)) {
			match = 1;
		} else {
			return 0;
		}
	}

	return match;
}



static int
filter(const char *node, const char *service,
		char **spoof_node, char **spoof_service,
		struct action *action)
{
	struct resolve_rule **rule;

	for (rule = condnet_config->resolve; *rule; rule++) {
		if (match(*rule, node, service)) {
			if (spoof_node && (*rule)->spoof_node) {
				free(*spoof_node);
				*spoof_node = strdup_or_null(
						(*rule)->spoof_node);
			}
			if (spoof_service && (*rule)->spoof_service) {
				free(*spoof_service);
				*spoof_service = strdup_or_null(
						(*rule)->spoof_service);
			}
			action_copy(action, &((*rule)->policy));
			return 1;
		}
	}

	return 0;
}



int
filter_resolve(const char *node, const char *service,
		char **real_node, char **real_service,
		struct action *action)
{
	action_copy(action, &(condnet_config->resolve_policy));

	if (node && real_node) {
		free(*real_node);
		*real_node = strdup_or_null(node);
	}
	if (service && real_service) {
		free(*real_service);
		*real_service = strdup_or_null(service);
	}

	if (condnet_config) {
		return filter(node, service, real_node, real_service,
				action);
	} else {
		return 0;
	}
}
