#ifndef RULES_H__
#define RULES_H__

#include <stdint.h>	/* uint16_t */
#include <stdio.h>	/* size_t */


struct action {
	int	ret;
	int	set_ret;
	int	err;
	int	set_err;
};


struct rules {
	struct action		in_policy;
	struct action		out_policy;
	struct action		resolve_policy;

	struct resolve_rule	**resolve;
	struct connection_rule	**in;
	struct connection_rule	**out;

	size_t			resolve_n;
	size_t			in_n;
	size_t			out_n;
};

struct resolve_rule {
	char		*node_pattern;
	char		*service_pattern;
	char		*spoof_node;
	char		*spoof_service;

	struct action	policy;

};

struct connection_rule {
	int		protocol;

	char		*from_addr;
	int		from_rev;
	uint16_t	from_port;
	char		*to_addr;
	int		to_rev;
	uint16_t	to_port;

	char		*spoof_addr;
	uint16_t	spoof_port;

	struct action	policy;
};


void rules_to_log(struct rules *rules);

struct rules *rules_new(void);
void rules_free(struct rules *rules);

void rules_add_inbound(struct rules *rules, struct connection_rule *rule);
void rules_add_outbound(struct rules *rules, struct connection_rule *rule);
void rules_add_resolve(struct rules *rules, struct resolve_rule *rule);

struct resolve_rule *rule_resolve_new(const char *node_pattern,
		const char *service_pattern,
		const char *spoof_node, const char *spoof_service,
		struct action *action);
struct connection_rule *rule_connection_new(int protocol,
		const char *from_addr, uint16_t from_port, int from_rev,
		const char *to_addr, uint16_t to_port, int to_rev,
		const char *spoof_addr, uint16_t spoof_port,
		struct action *action);

void action_set(struct action *policy, int ret, int err);
void action_copy(struct action *dest, struct action *src);
void action_zero(struct action *policy);

void action_to_str(char *buf, size_t len, struct action *action);


#endif
